package Dates;

/*
Un classe pour les entrée-sortie simplifiée en java, à but
pédagogique.

Copyright (2020) François Barthelemy

This library is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as
published by the Free Software Foundation; either version 2.1 of the
License, or (at your option) any later version.

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

The licence can be found at:

https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html#SEC4.
*/

import java.io.*;
/** La classe <code>Terminal</code> sert aux étudiants de l'UE NFA031
    pour leurs entrée/sorties textuelles dans le terminal. C'est un
    ensemble de méthodes statiques permettant de lire et écrire les
    types suivants: <code>String</code>, <code>int</code>,
    <code>bool</code>, <code>double</code> et <code>char</code>.

    <p>Par exemple si on compile le code suivant:</p>

<pre><code>public static void main (String args[]){
  int i1 = Terminal.lireInt();
  int i2 = Terminal.lireInt();
  Terminal.printInt(i1+i2);
}</code></pre>

<p>Et qu'à l'exécution l'utilisateur tape au clavier les caractères
suivants:</p>

<pre><code>23[entrée]
32[entrée]</code></pre>

<p> le programme affiche:</p>

<pre><code>66</code></pre>


<p> Remarque: Les entrées ne sont pas bufferisées. C'est-à-dire que
 chaque méthode consomme l'intégralité de la ligne tapée par
 l'utilisateur. Par ailleurs à l'exception de <code>lireChar</code>
 qui ignore les caractères en trop, toutes les autres méthodes exigent
 que tous les caractères de la ligne soient utilisés ppour reconnaître
 la valeur. Par exemple dans l'exemple ci-dessus si l'utilisateur tape:</p>

<pre><code>23a[entrée]</code></pre>

<p> le programme échoue (lève une exception) car la chaîne
<code>"23a"</code> ne correspond pas <i>dans son intégralité</i> comme
à valeur entière.</p>

 @author François Barthélemy
 @version 1.0
*/

public class Terminal{
	/** le reader sur stdin */
	static BufferedReader in =
        new BufferedReader(new InputStreamReader(System.in));

    /** constructeur privé */
    private Terminal(){} 

    /** Pour lire un <code>String</code> au clavier.
        @return la chaîne tapée jusqu'au premier retour à la ligne
        (<code>entrée</code>).
    
        @throws TerminalException si la lecture n'est pas possible.
    */
    public static String lireString()
    {
        String tmp="";
        char C='\0';
        try {
            tmp = in.readLine();
        }
        catch (IOException e)
            {
                exceptionHandler(e);
            }
        return tmp;
    } // fin de lireString()

    /** Pour lire un <code>int</code> au clavier.
        @return l'entier correspondant à la chaîne tapée. Retourne
        <code>0</code> si pas d'entier reconnaissable.
        @throws TerminalException si la lecture n'est pas possible (par exemple si on tape autre chose qu'un entier).*/
    public static int lireInt()
    {
        int x=0;
        try {
            x=Integer.parseInt(lireString());
        }
        catch (NumberFormatException e) {
            exceptionHandler(e);
        }
        return x ;
    }


    /** Pour lire un <code>boolean</code> au clavier.
        @return le booléen correspondant à la chaîne tapée. Retourne
        <code>true</code> si pas de booléen reconnaissable.
        @throws TerminalException  si la lecture n'est pas possible (par exemple si on tape autre chose que <code>true</code> ou <code>false</code>).*/
    public static boolean lireBoolean()  // Lire un entier
    {
        boolean b = true;
        try {
            b = Boolean.valueOf(lireString()).booleanValue();
        }
        catch (NumberFormatException e) {
            exceptionHandler(e);
        }
        return b;
    }

    /** Pour lire un <code>double</code> au clavier.
        @return le double correspondant à la chaîne tapée. Retourne
        <code>0.0</code> si pas de double reconnaissable.
        @throws TerminalException  si la lecture n'est pas possible (par exemple si on tape autre chose qu'un nombre.*/
    public static double lireDouble()  // Lire un double
    {
        double x=0.0;
        try {
            x=Double.valueOf(lireString()).doubleValue();
        }
        catch (NumberFormatException e) {
            exceptionHandler(e);
        }
        return x ;
    }

    /** Pour lire un <code>char</code> au clavier..
        
        Si l'utilsateur tape plusieurs caractère avant d'appuyer sur
        "entrée", le premier d'entre eux est retourné, et tous les
        autres sont ignorés.

        @return le caractère tapé. Si la lecture n'est pas possible (par exemple si on appuie sur entrée sans avoir tapé un seul caractère). */
    public static char lireChar()  // Lire un caractere
    {
        String tmp=lireString();
        if (tmp.length()==0)
            return '\n';
        else
            {
                return tmp.charAt(0);
            }
    }

    /** Pour écrire une chaîne de caractères dans le terminal.

        @param s la chaîne de caractère à écrire.

        @throws TerminalException si l'affichage n'est pas possible. */
    public static void ecrireString(String s){ // Afficher un String
        try{
            System.out.print(s);
        } catch (Exception ex){
            exceptionHandler(ex);
        }
    }

    /** Pour écrire une chaîne de caractères dans le terminal et
        ajouter un retour à la ligne.
        @param s la chaîne de caractère à écrire.

        @throws TerminalException si l'affichage n'est pas possible. */
    public static void ecrireStringln(String s) // Afficher un String
    {
        ecrireString(s);
        sautDeLigne();
    } // fin de ecrireStringln()

    /** Pour écrire un entier dans le terminal.
        @param i l'entier à écrire.
        @throws TerminalException si l'affichage n'est pas possible. */
    public static void ecrireInt(int i)  // Afficher un entier
    {
        ecrireString(""+i);
    }
    /** Pour écrire un entier dans le terminal et ajouter un retour à
        la ligne.
        @param i l'entier à écrire.
        @throws TerminalException si l'affichage n'est pas possible. */
    public static void ecrireIntln(int i)  // Afficher un entier
    {
        ecrireString(""+i);
        sautDeLigne();
    }

    /** Pour écrire un booléen dans le terminal.
        @param b le booléen à écrire.
        @throws TerminalException si l'affichage n'est pas possible. */
    public static void ecrireBoolean(boolean b){
        ecrireString(""+b);
    }

    /** Pour écrire un booléen dans le terminal et ajouter un retour
        à la ligne.
        @param b le booléen à écrire.
        @throws TerminalException si l'affichage n'est pas possible. */
    public static void ecrireBooleanln(boolean b){
        ecrireString(""+b);
        sautDeLigne();
    }

    /** Pour écrire un double dans le terminal.
        @param d le double à écrire.
        @throws TerminalException si l'affichage n'est pas possible. */
    public  static void ecrireDouble(double d)  // Afficher un double
    {
        ecrireString(""+d);
    }

    /** Pour écrire un double dans le terminal et ajouter un retour à
        la ligne. 
        @param d le double à écrire.
        @throws TerminalException si l'affichage n'est pas possible. */
    public  static void ecrireDoubleln(double d)  // Afficher un double
    {
        ecrireDouble(d);
        sautDeLigne();
    }

    /** Pour écrire un caractère dans le terminal.
        @param c le caractère à écrire.
        @throws TerminalException si l'affichage n'est pas possible. */
    public  static void ecrireChar(char c)  // Afficher un caractere
    {
        ecrireString(""+c);
    }

    /** Pour écrire un caractère dans le terminal et ajouter un retour
        à la ligne.
        @param c le caractère à écrire.
        @throws TerminalException si l'affichage n'est pas possible. */
    public  static void ecrireCharln(char c)  // Afficher un caractere
    {
        ecrireChar(c);
        sautDeLigne();
    }

    /** Pour écrire un retour à la ligne dans le terminal.
        @throws TerminalException si l'affichage n'est pas possible. */
    public static void sautDeLigne(){
        try{
            System.out.println();
        }catch(Exception ex){
            exceptionHandler(ex);
        }
    }

    /** <b>Fonction auxiliaire, ne pas utiliser.</b> Encapsule l'exception
     passée en argument dans une exception
     <code>TerminalException</code> et lève cette dernière.
     @param ex l'exception à encapsuler.
   */
    protected static void exceptionHandler(Exception ex){
        TerminalException err = new TerminalException(ex);
        throw err;
    }

    /** Pour afficher une exception dans le terminal.
        @param ex l'exception à afficher. */
    public static void ecrireException(Throwable ex){
        ecrireString(ex.toString());
        ex.printStackTrace(System.err);
    }
    /** lit au clavier une date au format jj/mm/aaaa. lève l'exception TerminalException
     * si la date est mal formée. Attention aucune vérification de validité de la date
     * n'est effectuée.
     * @return un tableau de 3 entiers représentant la date. */
	public static int[] lireDate()  {
		String s = lireString(); // Lit une ligne au clavier
		String [] ts = s.split("/"); // découpe selon les "/", mais retourne le tableau (!)
		if (ts.length != 3)	throw new TerminalException(new IOException());
		int t[] = new int[3];
		t[0] = Integer.valueOf(ts[2]); //annee
		t[1] = Integer.valueOf(ts[1]); // mois
		t[2] = Integer.valueOf(ts[0]); // jour
		return t;
	}
	/** demande un entier à l'utilisateur. Si il y a une erreur de saisi ou que
	 * l'entier n'est pas entre min et max, la question est reposée indéfiniment.
	 * 
	 * @param min borne min autorisée
	 * @param max borne max
	 * @return l'entier saisi par l'utilisateur.
	 */
	public static int demandeEntier(int min, int max) {
		int num = -1;
		while (num<0) { // On repose la question tant que pas bon
			try {
				int answer = Terminal.lireInt();
				if (answer>=min && answer <= max) {
					num=answer;
				}
			} catch (TerminalException e) {}
		}
		return num;
	}
}

/** Exception levée par les fonctions de la classe #Terminal */
class TerminalException extends RuntimeException{
    Exception ex;
    TerminalException(Exception e){
        ex = e;
    }
}

